<?php
// #######################################################
// Author: Markus Widmer
// Creation date: 17.07.2007
// Modification date: 08.04.2008
// Version: 0.2.0
// #######################################################



class TgExtra {

  // Global variables
  protected $rbac;
  protected $config;
  protected $connection;



  // -----------------------------------------------------
  // Constructor
  // Input: none
  // Output: object RBACcore
  // Description:
  //   Creates initial connections to the LDAP-server and
  //   sets some configuration parameters.
  // -----------------------------------------------------
  public function __construct( $inConfigurationFilename, $inRbacConfFile, $inRbacBase ) {

    $this->rbac = new RBAC( $inRbacConfFile, $inRbacBase );


    $this->config = new SimpleConfig( $inConfigurationFilename );


    // Create connection
    $this->connection['user'] = new LDAP();
    $this->connection['user']->connect( $this->config->getValue( "authentication", "host" ),
                                        $this->config->getValue( "authentication", "port" ),
                                        $this->config->getValue( "authentication", "version" ),
                                        preg_match( "/yes/i", $this->config->getValue( "authentication", "tls" ) ) ? true : false );
    $this->connection['user']->bind( $this->config->getValue( "authentication", "binddn" ),
                                     $this->config->getValue( "authentication", "password" ) );


    $this->connection['counter'] = new LDAP();
    $this->connection['counter']->connect( $this->config->getValue( "counter", "host" ),
                                           $this->config->getValue( "counter", "port" ),
                                           $this->config->getValue( "counter", "version" ),
                                           preg_match( "/yes/i", $this->config->getValue( "counter", "tls" ) ) ? true : false );
    $this->connection['counter']->bind( $this->config->getValue( "counter", "binddn" ),
                                        $this->config->getValue( "counter", "password" ) );


    $this->connection['resource'] = $this->rbac->getConnection( "resource" );
    $this->connection['role'] = $this->rbac->getConnection( "role" );

  }




  // -----------------------------------------------------
  // Function: userExists
  // Input: auth / xsd:string
  //        log / xsd:string
  //        username / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Checks if a given user exists in the LDAP directory.
  // -----------------------------------------------------
  function userExists( $inRequest ) {

    $result = new booleanResponse();  // The result


    $filter  = "(&" . $this->config->getValue( "authentication", "filter" );
    $filter .= "(" . $this->config->getValue( "authentication", "namingattribute" ) . "=" . $inRequest->username . "))";


    // Search for the users entry
    $arrUserEntry = $this->connection['user']->search( $this->config->getValue( "authentication", "base" ), $filter, "sub",
                                                       Array( $this->config->getValue( "authentication", "namingattribute" ) ) );


    if(    isset( $arrUserEntry[0] )
        && isset( $arrUserEntry[0]['dn'] ) ) {

      $result->result = true;

    }
    else {

      $result->result = false;

    }


    return $result;

  }



  // -----------------------------------------------------
  // Function: authenticate
  // Input: username / xsd:string
  //        password / xsd:string
  //        log / xsd:string
  // Output: sid / xsd:string
  // Description
  //   Tries to authenticate the user. If this is
  //   successful a session-ID is generated and a
  //   session is startet.
  // -----------------------------------------------------
  function authenticate( $inRequest ) {

    $filter = "";                          // search-filter
    $result = new authenticateResponse();  // service-resonse
    $arrUserEntry;                         // the users entry in the directory
    $intSid = "";                          // the generated session-ID
    $creationResult;                       // the result of the creation of the session


    // Construct the search-filter
    $filter .= "(&" . $this->config->getValue( "authentication", "filter" );
    $filter .= "(" . $this->config->getValue( "authentication", "namingattribute" ) . "=" . $inRequest->username . "))";


    // Search for the users entry
    $arrUserEntry = $this->connection['user']->search( $this->config->getValue( "authentication", "base" ), $filter, "sub",
                                                       Array( $this->config->getValue( "authentication", "namingattribute" ) ) );


    if(    isset( $arrUserEntry[0] )
        && isset( $arrUserEntry[0]['dn'] ) ) {

      // Try to bind with the given password
      $bindResult = $this->connection['user']->bind( $arrUserEntry[0]['dn'], $inRequest->password );


      if( $bindResult ) {

        $intSid = $this->createSessionID();


        // Try to create the session in the rbac-system
        $creationResult = $this->rbac->createSession( $inRequest->username, Array(), $intSid );
        if( $creationResult == $this->config->getValue( "errorCode", "OK" ) ) {

          $result->auth = $intSid;

        }
        else {

          return new SoapFault( "authenticationFault",
                                $this->config->getValue( "errorCode", "AUTHENTICATION_ERROR" ),
                                get_class( $this ),
                                $this->config->getValue( "errorDescription", "AUTHENTICATION_ERROR" ) );

        }

      }
      else {

        return new SoapFault( "authenticationFault",
                              $this->config->getValue( "errorCode", "AUTHENTICATION_ERROR" ),
                              get_class( $this ),
                              $this->config->getValue( "errorDescription", "AUTHENTICATION_ERROR" ) );

      }

    }
    else {

      return new SoapFault( "authenticationFault",
                            $this->config->getValue( "errorCode", "UNKNOWN_USER" ),
                            get_class( $this ),
                            $this->config->getValue( "errorDescription", "UNKNOWN_USER" ) );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: tgCheckAccess
  // Input: log / xsd:string
  //        session / xsd:string
  //        operation / xsd:string
  //        resource / xsd:string
  // Output: sid / xsd:string
  // Description
  //   Tries to authenticate the user. If this is
  //   successful a session-ID is generated and a
  //   session is startet.
  // -----------------------------------------------------
  public function tgCheckAccess( $inRequest ) {

    $result = new booleanResponse();


    $result->result = $this->rbac->checkAccess( $inRequest->auth, $inRequest->operation, $inRequest->resource );


    return $result;

  }




  // -----------------------------------------------------
  // Function: createSessionID
  // Input: none
  // Output: sid / xsd:string
  // Description
  //   Creates a new session-ID.
  // -----------------------------------------------------
  public function getSid() {

    $result = new getSidResponse();


    $result->sid = $this->createSessionID();


    return $result;

  }




  // -----------------------------------------------------
  // Function: filterBySid
  // Input: auth / xsd:string
  //        log / xsd:string
  //        sid / xsd:string
  //        resource / xsd:string
  //        operation / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Tries to authorize the user. If this is
  //   successful the function filters all the ressources
  //   given by checking if the sid has appropriate access.
  // -----------------------------------------------------
  function filterBySid( $inRequest ) {

    $filterBySidResult = Array();    // The resources that pass the filter
    $result = new filterResponse();  // The result


    $res = $inRequest->resource;
    // Make sure it is an array
    if( !is_array( $res ) ) {

      $res = Array( $res );

    }


    try {

      for( $i = 0; $i < sizeof( $res ); $i++ ) {

        if( $this->rbac->checkAccess( $inRequest->auth, $inRequest->operation, $res[$i] ) ) {

          $filterBySidResult[] = $res[$i];

        }

      }


      $result->resource = $filterBySidResult;

    }
    catch( RBACException $e ) {

      return new SoapFault( "rbacFault", $e->getCode(), get_class( $this->rbac ), $e->getMessage() );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: addMember
  // Input: auth / xsd:string
  //        log / xsd:string
  //        username / xsd:string
  //        role / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Tries to add a user to a project. This is only possible
  //   if the user exists and the session has the permission
  //   "delegate" on the project.
  // -----------------------------------------------------
  public function addMember( $inRequest ) {

    $arrSplit = Array();
    $project = false;
    $result = new booleanResponse();  // The return-result


    // Extract the project from the role
    $arrSplit = preg_split( "/[,]/", $inRequest->role );

    for( $i = 0; $i < sizeof( $arrSplit ); $i++ ) {

      if( preg_match( "/^tgpr[0-9]+$/i", $arrSplit[$i] ) ) {

        $project = $arrSplit[$i];

      }

    }


    if(    $project
        && $this->rbac->checkAccess( $inRequest->auth, "delegate", $project ) ) {

      try {

        if( $this->rbac->assignUser( $inRequest->username, $inRequest->role ) ) {

          $result->result = true;

        }
        else {

          $result->result = false;

        }

      }
      catch( RBACException $e ) {

        return new SoapFault( "rbacFault", $e->getCode(), get_class( $this->rbac ), $e->getMessage() );

      }

    }
    else {

      $result->result = false;

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: tgGrantPermission
  // Input: auth / xsd:string
  //        log / xsd:string
  //        role / xsd:string
  //        resource / xsd:string
  //        operation / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Grants a permission to a resource if the user
  //   given by the auth parameter has the right to
  //   do this.
  // -----------------------------------------------------
  public function tgGrantPermission( $inRequest ) {

    $result = new booleanResponse();


    try {

      if( $this->rbac->checkAccess( $inRequest->auth, "delegate", $inRequest->resource ) ) {

        $result->result = $this->rbac->grantPermission( $inRequest->resource, $inRequest->operation, $inRequest->role );

      }
      else {

        return new SoapFault( "authenticationFault",
                               $this->config->getValue( "errorCode", "INSUFFICIENT_ACCESS" ),
                               get_class( $this ),
                               $this->config->getValue( "errorDescription", "INSUFFICIENT_ACCESS" ) );

      }

    }
    catch( RBACException $e ) {

      return new SoapFault( "rbacFault", $e->getCode(), get_class( $this->rbac ), $e->getMessage() );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: tgRevokePermission
  // Input: auth / xsd:string
  //        log / xsd:string
  //        role / xsd:string
  //        resource / xsd:string
  //        operation / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Revokes a permission for a resource if the user
  //   given by the auth parameter has the right to
  //   do this.
  // -----------------------------------------------------
  public function tgRevokePermission( $inRequest ) {

    $result = new booleanResponse();


    try {

      if( $this->rbac->checkAccess( $inRequest->auth, "delegate", $inRequest->resource ) ) {

        $result->result = $this->rbac->revokePermission( $inRequest->operation, $inRequest->resource, $inRequest->role );

      }
      else {

        return new SoapFault( "authenticationFault",
                               $this->config->getValue( "errorCode", "INSUFFICIENT_ACCESS" ),
                               get_class( $this ),
                               $this->config->getValue( "errorDescription", "INSUFFICIENT_ACCESS" ) );

      }

    }
    catch( RBACException $e ) {

      return new SoapFault( "rbacFault", $e->getCode(), get_class( $this->rbac ), $e->getMessage() );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: getObjects
  // Input: auth / xsd:string
  //        log / xsd:string
  //        project / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Returns a list of all resources that the user corresponding
  //   to the session-ID (auth) may read.
  // -----------------------------------------------------
  public function getObjects( $inRequest ) {

    $result = new resourcesetResponse();  // The return-result
    $filter = "";                         // RBAC-filter
    $arrResource = Array();               // Resoult of the RBAC-search
    $i = 0;                               // Loop


    $filter  = "(&" . $this->rbac->getConfiguration()->getValue( "resource", "filter" );
    $filter .= "(tgprojectid=" . $inRequest->project . "))";


    $arrResource = $this->connection['resource']->search( $this->rbac->getConfiguration()->getValue( "resource", "base" ),
                                                          $filter, "sub",
                                                          Array( $this->rbac->getConfiguration()->getValue( "resource", "aliasattribute" ),
                                                          $this->rbac->getConfiguration()->getValue( "resource", "namingattribute" ) ) );


    for( $i = 0; $i < sizeof( $arrResource ); $i++ ) {

      if( $this->rbac->checkAccess( $inRequest->auth, "read",
                                    $arrResource[$i][$this->rbac->getConfiguration()->getValue( "resource", "aliasattribute" )][0] ) ) {

        $result->resource[] = $arrResource[$i][$this->rbac->getConfiguration()->getValue( "resource", "aliasattribute" )][0];

      }

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: isPublic
  // Input: auth / xsd:string
  //        log / xsd:string
  //        resource / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Returns true if the resource is public. In every
  //   other case there will be returned false.
  // -----------------------------------------------------
  public function isPublic( $inRequest ) {

    $result = new booleanResponse();
    $filter = "";
    $arrResource = Array();


    // Create a filter that searches for the
    // given resource.
    $filter  = "(&" . $this->rbac->getConfiguration()->getValue( "resource", "filter" );
    $filter .= "(|(" . $this->rbac->getConfiguration()->getValue( "resource", "namingattribute" ) . "=" . $inRequest->resource . ")";
    $filter .= "  (" . $this->rbac->getConfiguration()->getValue( "resource", "aliasattribute" ) . "=" . $inRequest->resource . ")))";


    $arrResource = $this->connection['resource']->search( $this->rbac->getConfiguration()->getValue( "resource", "base" ),
                                                          $filter, "sub",
                                                          Array( "tgispublic" ) );


    // If the attribute is not set or there was no
    // resource found, return false.
    if( !isset( $arrResource[0]['tgispublic'] ) ) {

      $result->result = false;

    }
    else {

      preg_match( "/^true$/i", $arrResource[0]['tgispublic'][0] ) ? $result->result = true : $result->result = false;

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: getOwner
  // Input: auth / xsd:string
  //        log / xsd:string
  //        resource / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Returns the owner of a resource. This owner has nothing
  //   to do with any permissions, it is just the owner.
  // -----------------------------------------------------
  public function getOwner( $inRequest ) {

    $result = new getOwnerResponse();  // The return-result
    $filter = "";                      // RBAC-filter
    $arrResource = Array();            // Resoult of the RBAC-search


    // Create a filter that searches for the
    // given resource.
    $filter  = "(&" . $this->rbac->getConfiguration()->getValue( "resource", "filter" );
    $filter .= "(|(" . $this->rbac->getConfiguration()->getValue( "resource", "namingattribute" ) . "=" . $inRequest->resource . ")";
    $filter .= "  (" . $this->rbac->getConfiguration()->getValue( "resource", "aliasattribute" ) . "=" . $inRequest->resource . ")))";


    if( $this->rbac->checkAccess( $inRequest->auth, "read", $inRequest->resource ) ) {

      $arrResource = $this->connection['resource']->search( $this->rbac->getConfiguration()->getValue( "resource", "base" ),
                                                            $filter, "sub",
                                                            Array( "tgresourceowner" ) );


      if( sizeof( $arrResource ) == 1 ) {

        $result->owner = $arrResource[0]['tgresourceowner'][0];

      }

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: getMembers
  // Input: auth / xsd:string
  //        log / xsd:string
  //        project / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Returns a list of members in a project.
  // -----------------------------------------------------
  public function getMembers( $inRequest ) {

    $result = new usersetResponse();  // The return-result
    $arrMember = Array();             // Resoult of the RBAC-search


    $arrMember = $this->rbac->authorizedUsers( $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=" . $inRequest->project . ","
                                               . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                               . $this->config->getValue( "project", "base" )
                                               . "," . $this->rbac->getConfiguration()->getValue( "role", "base" ) );


    // The user has to be in the project to be
    // allowed to display all other users
    if( in_array( $this->rbac->sessionUser( $inRequest->auth ), $arrMember ) ) {

      $result->username = $arrMember;

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: getRights
  // Input: auth / xsd:string
  //        log / xsd:string
  //        resource / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Returns a list of operations allowed by the user
  //   on a specific resource.
  // -----------------------------------------------------
  public function getRights( $inRequest ) {

    $arrOperation = Array();
    $result = new operationsetResponse();  // The return-result


    if( preg_match( "/.+/", $inRequest->username ) ) {

      if( $this->rbac->checkAccess( $inRequest->auth, "delegate", $inRequest->resource ) ) {

        $arrOperation = $this->rbac->userOperationsOnObject( $inRequest->username, $inRequest->resource );

      }
      else {

        return new SoapFault( "authenticationFault",
                               $this->config->getValue( "errorCode", "INSUFFICIENT_ACCESS" ),
                               get_class( $this ),
                               $this->config->getValue( "errorDescription", "INSUFFICIENT_ACCESS" ) );

      }

    }
    else {

      $arrOperation = $this->rbac->userOperationsOnObject( $this->rbac->sessionUser( $inRequest->auth ), $inRequest->resource );

    }


    $result->operation = $arrOperation;


    return $result;

  }




  // -----------------------------------------------------
  // Function: publish
  // Input: auth / xsd:string
  //        log / xsd:string
  //        project / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Returns a list of operations allowed by the user
  //   on a specific resource.
  // -----------------------------------------------------
  public function publish( $inRequest ) {

    $arrResource = Array();
    $arrModify = Array();
    $filter = "";
    $result = new booleanResponse();  // The return-result


    $filter  = "(&" . $this->rbac->getConfiguration()->getValue( "resource", "filter" );
    $filter .= "(|(" . $this->rbac->getConfiguration()->getValue( "resource", "namingattribute" ) . "=" . $inRequest->resource . ")";
    $filter .= "  (" . $this->rbac->getConfiguration()->getValue( "resource", "aliasattribute" ) . "=" . $inRequest->resource . ")))";


    // Get the resource
    $arrResource = $this->connection['resource']->search( $this->rbac->getConfiguration()->getValue( "resource", "base" ), $filter, "sub",
                                                          Array( "tgispublic", "tgprojectid" ) );


    if( $this->rbac->checkAccess( $inRequest->auth, "publish", $inRequest->resource ) ) {

      $arrModify['tgispublic'][] = "TRUE";
      $result->result = $this->connection['resource']->modify( $arrResource[0]['dn'], $arrModify );

    }
    else {

      return new SoapFault( "authenticationFault",
                             $this->config->getValue( "errorCode", "INSUFFICIENT_ACCESS" ),
                             get_class( $this ),
                             $this->config->getValue( "errorDescription", "INSUFFICIENT_ACCESS" ) );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: getProjectDescription
  // Input: auth / xsd:string
  //        log / xsd:string
  //        project / xsd:string
  // Output: result / xsd:string
  // Description
  //   Returns the description of a project
  // -----------------------------------------------------
  public function getProjectDescription( $inRequest ) {

    $result = new getProjectDescriptionResponse();
    $filter = "";


    $filter  = "(&" . $this->rbac->getConfiguration()->getValue( "role", "filter" );
    $filter .= "(" . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=" . $inRequest->project . "))";


    $arrProject = $this->connection['role']->getEntry( $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                                       . $inRequest->project . ","
                                                       . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                                       . $this->config->getValue( "project", "base" )
                                                       . "," . $this->rbac->getConfiguration()->getValue( "role", "base" ) );


    if( isset( $arrProject['dn'] ) ) {

      $result->project = new stdClass();
      $result->project->description = $arrProject['tgprojectdescription'][0];
      $result->project->name = $arrProject['tgprojectname'][0];
      $result->project->id = $arrProject['tgprojectid'][0];

    }
    else {

      $result->project->description = "Not available";

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: tgAssignedProjects
  // Input: auth / xsd:string
  //        log / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Returns a list of all projects of the user corresponding
  //   to the session-ID (auth).
  // -----------------------------------------------------
  public function tgAssignedProjects( $inRequest ) {

    $filter = "";
    $username = "";
    $arrFound = Array();
    $result = new rolesetResponse();  // The return-result
    $i = 0;                           // Loop
    $j = 0;                           // Loop


    // By default the result is an empty array
    $result->role = Array();


    // The user corresponding to the session
    $username = $this->rbac->sessionUser( $inRequest->auth );


    // Search all roles in which the user is performer.
    $filter  = "(&" . $this->rbac->getConfiguration()->getValue( "role", "filter" );
    $filter .= "(rbacPerformer=" . $username . "))";

    $arrRole = $this->connection['role']->search( $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                                  . $this->config->getValue( "project", "base" ) . ","
                                                  . $this->rbac->getConfiguration()->getValue( "role", "base" ),
                                                  $filter, "sub", Array( $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) ) );


    for( $i = 0; $i < sizeof( $arrRole ); $i++ ) {

      $arrSplit = preg_split( "/[,]/", $arrRole[$i]['dn'] );
      $projectBelong = "";


      for( $j = 0; $j < sizeof( $arrSplit ); $j++ ) {

        if( preg_match( "/^TGPR[0-9]+$/i",
                        preg_replace( "/^" . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=/i", "", $arrSplit[$j] ) ) ) {

          $projectBelong = preg_replace( "/^" . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=/i", "", $arrSplit[$j] );

        }

      }


      if(    preg_match( "/.+/", $projectBelong )
          && !isset( $arrFound[strtolower( $projectBelong )] ) ) {

        $arrFound[strtolower( $projectBelong )] = 1;

      }

    }


    foreach( $arrFound as $key => $value ) {

      $result->role[] = strtoupper( $key );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: deleteMember
  // Input: auth / xsd:string
  //        log / xsd:string
  //        username / xsd:string
  //        role / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Tries to remove a user from a role. This is only possible
  //   if the user exists and the session has the permission
  //   "delegate" on the project.
  // -----------------------------------------------------
  public function deleteMember( $inRequest ) {

    $arrSplit = Array();
    $project = false;
    $result = new booleanResponse();  // The return-result


    // Extract the project from the role
    $arrSplit = preg_split( "/[,]/", $inRequest->role );

    for( $i = 0; $i < sizeof( $arrSplit ); $i++ ) {

      if( preg_match( "/^tgpr[0-9]+$/i", $arrSplit[$i] ) ) {

        $project = $arrSplit[$i];

      }

    }


    if(    $project
        && $this->rbac->checkAccess( $inRequest->auth, "delegate", $project ) ) {

      if( $this->rbac->deassignUser( $inRequest->username, $inRequest->role ) ) {

        $result->result = true;

      }
      else {

        $result->result = false;

      }

    }
    else {

      $result->result = false;

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: unregisterResource
  // Input: auth / xsd:string
  //        log / xsd:string
  //        project / xsd:string
  //        uri / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Tries to remove a resource from the directory. There for
  //   a user has to have the permission "delete" on the
  //   resource.
  // -----------------------------------------------------
  function unregisterResource( $inRequest ) {

    $resourceNamingAttribute = "";    // The naming-attribute of the resource
    $resourceAliasAttribute = "";     // The alias-attribute of the resource
    $filter = "";                     // The LDAP-filter to find the resource
    $result = new booleanResponse();  // The return-result


    if( $this->rbac->checkAccess( $inRequest->auth, "delete", $inRequest->uri ) ) {

      // The TextGrid-resource naming-attribute
      $resourceNamingAttribute = $this->rbac->getConfiguration()->getValue( "resource", "namingattribute" );
      $resourceAliasAttribute = $this->rbac->getConfiguration()->getValue( "resource", "aliasattribute" );


      $filter  = "(&" . $this->rbac->getConfiguration()->getValue( "resource", "filter" );
      $filter .= "(|(" . $resourceNamingAttribute . "=" . $inRequest->uri . ")";
      $filter .= "  (" . $resourceAliasAttribute . "=" . $inRequest->uri . ")))";

      $arrResource = $this->connection['resource']->search( $this->rbac->getConfiguration()->getValue( "resource", "base" ), $filter, "sub" );


      if( sizeof( $arrResource ) == 1 ) {

        $result->result = $this->connection['resource']->delete( $arrResource[0]['dn'] );

      }
      else {

        return new SoapFault( "unknownResourceFault",
                              $this->config->getValue( "errorCode", "RESOURCENOTFOUND_ERROR" ),
                              get_class( $this ),
                              $this->config->getValue( "errorDescription", "RESOURCENOTFOUND_ERROR" ) );

      }

    }
    else {

      $result->result = false;

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: registerResource
  // Input: auth / xsd:string
  //        log / xsd:string
  //        project / xsd:string
  //        uri / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Tries to add a resource to the directory. There for
  //   a user has to have the permission "create" on the
  //   resource "project".
  // -----------------------------------------------------
  function registerResource( $inRequest ) {

    $connection = false;              // The connection for resources
    $registered = false;              // The result of the registration-tries
    $registerTry = 10;                // The number of tries of registering the resource
    $resourceNamingAttribute = "";    // The naming-attribute of the resource
    $arrCounter = false;              // The counter entry
    $result = new booleanResponse();  // The return-result


    // The TextGrid-resource naming-attribute
    $resourceNamingAttribute = $this->rbac->getConfiguration()->getValue( "resource", "namingattribute" );


    // Look for the counter entry for resources.
    $arrCounter = $this->connection['counter']->getEntry( "cn=resource," . $this->config->getValue( "counter", "base" ) );


    // If there is a counter entry, use it
    if( isset( $arrCounter['dn'] ) ) {

      $freeNumber = $arrCounter['sn'][0] + 1;

    }
    else {

      $arrResourceName = $this->connection['resource']->search( $this->config->getValue( "textGridResource", "base" ),
                                                                "(objectClass=textGridResource)",
                                                                "one", $resourceNamingAttribute );


      // Every returned resource has to be examined for
      // its number to get the next free one.
      for( $i = 0; $i < sizeof( $arrResourceName ); $i++ ) {

        $resourceName = preg_replace( "/^TGRS/i", "", $arrResourceName[$i][$resourceNamingAttribute][0] );
        $maxNumber = max( $maxNumber, intval( $resourceName ) );

      }


      $freeNumber = $maxNumber + 1;

    }


    if( $this->rbac->checkAccess( $inRequest->auth, "create", $inRequest->project ) ) {

      // As long as the resource is not registered and the number
      // of tries has not been reached, try to register
      while(    !$registered
             && ($registerTry > 0) ) {

        // This is the resource-entry that will be
        // added to the directory
        $arrEntry = Array();
        $arrEntry['objectclass'][] = "textgridResource";
        $arrEntry['objectclass'][] = "rbacResource";
        $arrEntry['rbacoperation'][] = "read";
        $arrEntry['rbacoperation'][] = "write";
        $arrEntry['rbacoperation'][] = "delegate";
        $arrEntry['rbacoperation'][] = "delete";
        $arrEntry['rbacoperation'][] = "publish";
        $arrEntry['tgresourceuri'][] = $inRequest->uri;
        $arrEntry['tgprojectid'][] = $inRequest->project;
        $arrEntry['tgispublic'][] = "FALSE";
        $arrEntry[$resourceNamingAttribute][] = "TGRS" . ($freeNumber);
        $arrEntry['tgresourceowner'][] = $this->rbac->sessionUser( $inRequest->auth );


        // Add the default permissions to the resource
        $arrEntry['rbacpermission'][] = $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=Projektleiter,"
                                        . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=" . $inRequest->project . ","
                                        . $this->rbac->getConfiguration()->getValue( "project", "base" ) . ":-:delegate";
        $arrEntry['rbacpermission'][] = $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=Projektleiter,"
                                        . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=" . $inRequest->project . ","
                                        . $this->rbac->getConfiguration()->getValue( "project", "base" ) . ":-:publish";
        $arrEntry['rbacpermission'][] = $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=Administrator,"
                                        . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=" . $inRequest->project . ","
                                        . $this->rbac->getConfiguration()->getValue( "project", "base" ) . ":-:delete";
        $arrEntry['rbacpermission'][] = $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=Bearbeiter,"
                                        . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=" . $inRequest->project . ","
                                        . $this->rbac->getConfiguration()->getValue( "project", "base" ) . ":-:read";
        $arrEntry['rbacpermission'][] = $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=Bearbeiter,"
                                        . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=" . $inRequest->project . ","
                                        . $this->rbac->getConfiguration()->getValue( "project", "base" ) . ":-:write";
        $arrEntry['rbacpermission'][] = $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=Beobachter,"
                                        . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=" . $inRequest->project . ","
                                        . $this->rbac->getConfiguration()->getValue( "project", "base" ) . ":-:read";


        // Try to add the resource with the appropriate number. If
        // this fails, we will try again (10 times).
        $registered = $this->connection['resource']->add( $this->rbac->getConfiguration()->getValue( "resource", "namingattribute" ) . "=TGRS"
                                                          . $freeNumber . "," . $this->config->getValue( "textGridResource", "base" ),
                                                          $arrEntry );


        $registerTry--;


        $registered ? false : $freeNumber++;

      }


      // This entry will add or modify the directory that way
      // that there is a counter entry.
      $arrEntry = Array();
      $arrEntry['objectclass'][] = "person";
      $arrEntry['cn'][] = "resource";
      $arrEntry['sn'][] = $freeNumber;


      if( !isset( $arrCounter['dn'] ) ) {

        $this->connection['counter']->add( "cn=resource," . $this->config->getValue( "counter", "base" ), $arrEntry );

      }
      else {

        $this->connection['counter']->modify( "cn=resource," . $this->config->getValue( "counter", "base" ), $arrEntry );

      }


      $result->result = $registered;

    }
    else {

      $result->result = false;

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: getLeader
  // Input: log / xsd:string
  // Output: username[] / tns:xsd:string
  // Description
  //   Searches for the leader(s) of a project.
  // -----------------------------------------------------
  public function getLeader( $inRequest ) {

    $arrUser = Array();
    $result = new usersetResponse();


    $result->username = $this->rbac->assignedUsers( "Projektleiter," . $inRequest->project . ","
                                                    . $this->config->getValue( "project", "base" ) );


    return $result;

  }




  // -----------------------------------------------------
  // Function: getAllProjects
  // Input: log / xsd:string
  // Output: project[] / tns:projectInfo
  // Description
  //   Searches for all projects and returns them as a list.
  // -----------------------------------------------------
  public function getAllProjects( $inRequest ) {

    $arrProject = Array();                    // All project-entries found
    $connection = false;                    // The connection for roles
    $result = new getAllProjectsResponse();  // Return


    // The role-connection is needed because projects
    // are representated as roles.
    $connection = $this->rbac->getConnection( "resource" );


    // Search the directory
    $arrProject = $connection->search( $this->rbac->getConfiguration()->getValue( "project", "base" ), "(objectClass=rbacRole)", "one",
                                       $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) );


    for( $i = 0; $i < sizeof( $arrProject ); $i++ ) {

      $result->project[] = new ProjectInfo( $arrProject[$i][$this->rbac->getConfiguration()->getValue( "role", "namingattribute" )][0],
                                            $arrProject[$i]['tgprojectname'][0], $arrProject[$i]['tgprojectdescription'][0] );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: createProject
  // Input: auth / xsd:string
  //        log / xsd:string
  //        name / xsd:string
  //        description / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Tries to authorize the user. If this is
  //   successful the given role is added to the system.
  //   This function creates roles without a hirarchy, so it
  //   only has to check if there is access to the "role_base".
  // -----------------------------------------------------
  function createProject( $inRequest ) {

    $createProjectResult = false;           // The result of the process
    $connection = false;                    // The connection to the projects
    $arrProjectName = Array();              // All present projects
    $projectName = "";                      // At last Holds the project-name
    $maxNumber = 0;                         // The highest free project-number
    $result = new createProjectResponse();  // The result
    $i = 0;                                 // Loop


    if( $this->rbac->checkAccess( $inRequest->auth, "registerResource", "resource_base" ) ) {

      // The role-connection is needed because projects
      // are representated as roles.
      $connection = $this->rbac->getConnection( "role" );


      // Search the directory
      $arrProjectName = $connection->search( $this->rbac->getConfiguration()->getValue( "project", "base" ), "(objectClass=rbacRole)", "one",
                                             $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) );


      // Every returned project has to be examined for
      // its number to get the next free one.
      for( $i = 0; $i < sizeof( $arrProjectName ); $i++ ) {

        $projectName = preg_replace( "/^TGPR/i", "", $arrProjectName[$i][$this->rbac->getConfiguration()->getValue( "role", "namingattribute" )][0] );
        $maxNumber = max( $maxNumber, intval( $projectName ) );

      }


      try {

        // Create the project-role
        $createProjectResult = $this->rbac->addAscendant( "TGPR" . ($maxNumber + 1),
                                                          $this->rbac->getConfiguration()->getValue( "project", "base" ) );


        // The user of the session-ID will be the Leader
        // of this new project.
        $this->rbac->assignUser( $this->rbac->sessionUser( $inRequest->auth ),
                                 $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=Projektleiter,"
                                 . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=TGPR" . ($maxNumber + 1) . ","
                                 . $this->rbac->getConfiguration()->getValue( "project", "base" ) );


        // Set the project-description
        $connection->modify( $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "=TGPR" . ($maxNumber + 1) . ","
                             . $this->rbac->getConfiguration()->getValue( "project", "base" ),
                             Array( "tgprojectdescription" => Array( $inRequest->description ) ) );


        $result->projectId = "TGPR" . ($maxNumber + 1);

      }
      catch( RBACException $e ) {

        return new SoapFault( "rbacFault", $e->getCode(), get_class( $this->rbac ), $e->getMessage() );

      }

    }
    else {

      return new SoapFault( "authenticationFault",
                            $this->config->getValue( "errorCode", "INSUFFICIENT_ACCESS" ),
                            get_class( $this ),
                            $this->config->getValue( "errorDescription", "INSUFFICIENT_ACCESS" ) );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: deactivateProject
  // Input: auth / xsd:string
  //        log / xsd:string
  //        project / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Adds an assigned role to the list of active session
  //   roles.
  // -----------------------------------------------------
  public function deactivateProject( $inRequest ) {

    $renameResult = false;            // The result of the rename process
    $flagResult = false;              // The result of setting the flag to the project role
    $result = new booleanResponse();  // Return
    $arrProject = "";                  // The username corresponding to the session-ID


    if( $this->rbac->checkAccess( $inRequest->auth, "delegate", $inRequest->project ) ) {

      $arrProject = $this->connection['role']->getEntry( $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                                         . $inRequest->project . ","
                                                         . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                                         . $this->config->getValue( "project", "base" )
                                                         . "," . $this->rbac->getConfiguration()->getValue( "role", "base" ) );


      // Mark all users as deactivated that are directly
      // assigned to the project role.
      for( $i = 0; $i < sizeof( $arrModify['rbacperformer'] ); $i++ ) {

        $arrModify['rbacperformer'][] = $arrProject['rbacperformer'][$i] . "__deactivated";

      }


      // Try to rename all users so they cannot
      // activate any roles from this project
      // anymore.
      $renameResult = $this->renameRbacPerformers( $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                   . $inRequest->project . ","
                                   . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                   . $this->config->getValue( "project", "base" )
                                   . "," . $this->rbac->getConfiguration()->getValue( "role", "base" ) );


      if( $renameResult ) {

        // Set the flag to "TRUE"
        $arrModify['tgprojectdeactivated'][] = "TRUE";


        $flagResult = $this->connection['role']->modify( $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                         . $inRequest->project . ","
                                         . $this->rbac->getConfiguration()->getValue( "role", "namingattribute" ) . "="
                                         . $this->config->getValue( "project", "base" )
                                         . "," . $this->rbac->getConfiguration()->getValue( "role", "base" ),
                                         $arrModify );
      }


      $result->result = $renameResult && $flagResult;

    }
    else {

      return new SoapFault( "authenticationFault",
                            $this->config->getValue( "errorCode", "INSUFFICIENT_ACCESS" ),
                            get_class( $this ),
                            $this->config->getValue( "errorDescription", "INSUFFICIENT_ACCESS" ) );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: renameRbacPerformers
  // Input: inBase / string
  //        inRecursive / boolean
  // Output: result / xsd:boolean
  // Description
  //   The users stored in the rbacPerformer attribute are
  //   recursively renamed to <username>__deactivated.
  // -----------------------------------------------------
  private function renameRbacPerformers( $inBase, $inRecursive = true ) {

    $arrEntry = Array();
    $arrModify = Array();
    $i = 0;
    $result = true;


    // Get the entry
    $arrEntry = $this->connection['role']->getEntry( $inBase );


    // Mark all users as deactivated that are directly
    // assigned to the project role.
    for( $i = 0; $i < sizeof( $arrEntry['rbacperformer'] ); $i++ ) {

      $arrModify['rbacperformer'][] = $arrEntry['rbacperformer'][$i] . "__deactivated";

    }


    // If there are any users present in the entry,
    // send the modifications.
    if( sizeof( $arrModify['rbacperformer'] ) > 0 ) {

      $this->connection['role']->modify( $inBase, $arrModify );

    }


    // Query all sub entries that still need to be
    // processed.
    $arrSub = $this->connection['role']->search( $inBase, $this->rbac->getConfiguration()->getValue( "role", "filter" ), "one" );


    for( $i = 0; $i < sizeof( $arrSub ); $i++ ) {

      $result = $result && $this->renameRbacPerformers( $arrSub[$i]['dn'] );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: tgAddActiveRole
  // Input: auth / xsd:string
  //        log / xsd:string
  //        role / xsd:string
  // Output: result / xsd:boolean
  // Description
  //   Adds an assigned role to the list of active session
  //   roles.
  // -----------------------------------------------------
  public function tgAddActiveRole( $inRequest ) {

    $result = new booleanResponse();  // Return
    $username = "";                   // The username corresponding to the session-ID


    $username = $this->rbac->sessionUser( $inRequest->auth );


    if( preg_match( "/.+/", $username ) ) {

      $result->result = $this->rbac->addActiveRole( $username, $inRequest->auth, $inRequest->role );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: tgDropActiveRole
  // Input: auth / xsd:string
  //        log / xsd:string
  //        role / xsd:string
  // Output: sid / string
  // Description
  //   Removes a role from the list of active session roles.
  // -----------------------------------------------------
  public function tgDropActiveRole( $inRequest ) {

    $result = new booleanResponse();  // Return
    $username = "";                   // The username corresponding to the session-ID


    $username = $this->rbac->sessionUser( $inRequest->auth );


    if( preg_match( "/.+/", $username ) ) {

      $result->result = $this->rbac->dropActiveRole( $username, $inRequest->auth, $inRequest->role );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: tgAssignedRoles
  // Input: auth / xsd:string
  //        log / xsd:string
  //        username / xsd:string
  // Output: sid / string
  // Description
  //   Creates a random string containing characters and
  //   numbers.
  // -----------------------------------------------------
  public function tgAssignedRoles( $inRequest ) {

    $result = new rolesetResponse();
    $arrActiveRole = Array();
    $arrProject = Array();
    $arrAllreadyChecked = Array();
    $connection = false;
    $i = 0;  // Loop


    if(    preg_match( "/.+/", $inRequest->username )
        && !preg_match( "/^" . $inRequest->username . "$/i", $this->rbac->sessionUser( $inRequest->auth ) ) ) {

      // The roles of the user
      $arrRole = $this->rbac->assignedRoles( $inRequest->username );


      // Extract the different projects the user is
      // assigned by his roles.
      for( $i = 0; $i < sizeof( $arrRole ); $i++ ) {

        $arrSplit = preg_split( "/[,]/", $arrRole[$i] );

        for( $j = 0; $j < sizeof( $arrSplit ); $j++ ) {

          if(    preg_match( "/TGPR[0-9]+/i", $arrSplit[$j] )
              && !in_array( trim( $arrSplit[$j] ), $arrProject ) ) {

            $arrProject[] = trim( $arrSplit[$j] );
            break;

          }

        }

      }


      // For each project the username is in, check if the
      // session-ID has the right to display the assigned roles.
      for( $i = 0; $i < sizeof( $arrProject ); $i++ ) {

        if( $this->rbac->checkAccess( $inRequest->auth, "delegate", $arrProject[$i] ) ) {

          for( $j = 0; $j < sizeof( $arrRole ); $j++ ) {

            if( preg_match( "/" . $arrProject[$i] . "/i", $arrRole[$j] ) ) {

              $result->role[] = $arrRole[$j];

            }

          }

        }

      }

    }
    else {

      $result->role = $this->rbac->assignedRoles( $this->rbac->sessionUser( $inRequest->auth ) );

    }


    return $result;

  }




  // -----------------------------------------------------
  // Function: createSessionID
  // Input: none
  // Output: sid / string
  // Description
  //   Creates a random string containing characters and
  //   numbers.
  // -----------------------------------------------------
  private function createSessionID() {

    $sid = "";  // Session-ID
    $tmp = "";  // Temporaere Session-ID


    // Setzen eines Startwerts fuer den Zufallsgenerator
    mt_srand( (double)microtime() * 1000000 );


    // Erzeugen eines Zufallsstrings
    for( $i = 0; $i < 256; $i++ ) {

      $tmp = chr( mt_rand( 0, 255 ) );


      if( preg_match( "/[a-zA-Z0-9]/", $tmp ) ) {

        $sid .= $tmp;

      }

    }


    return $sid;

  }

}
?>
